//
//  DiceGL.m
//  DiceView
//
//  Created by kunii on 10/03/29.
//  Copyright 2010 Apple Inc. All rights reserved.
//

#import "DiceGL.h"

@implementation DiceGL

/*
	UIImageをテクスチャにマッピングする。
*/
-(void)map:(UIImage*)inImage texture:(GLuint)inTexture {
	CGImageRef cgImage = [inImage CGImage];
	int width = CGImageGetWidth(cgImage);
	int height = CGImageGetHeight(cgImage);
	CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
	void* data = malloc(width * height * 4);
	CGContextRef cgContext = CGBitmapContextCreate(data, width, height, 8, width * 4, colorSpace, kCGImageAlphaNoneSkipLast | kCGBitmapByteOrder32Big);
	if (cgContext != NULL) {
		CGContextDrawImage(cgContext, CGRectMake(0.0f, 0.0f, width, height), cgImage);		
		glBindTexture(GL_TEXTURE_2D, inTexture);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
		GLenum err = glGetError();
		if (err != GL_NO_ERROR)
			NSLog(@"Error uploading texture. glError: 0x%04X", err);
		CGContextRelease(cgContext);
	}	
	free(data);
	CGColorSpaceRelease(colorSpace);
}

/*
	初期画像（サイコロ）にする。
*/
-(void)initImage {
	for (int i = 0; i < 6; i++) {
		[self map:defaultImages[i] texture:texture[i]];
	}
}

/*
	初期化
	
	初期画像（サイコロ）、テクスチャの用意。
*/
-(id)init {
	if ([super init]) {
		for (int i = 0; i < 6; i++) {
			defaultImages[i] = [UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[NSString stringWithFormat:@"%d", i + 1] ofType:@"png"]];
			[defaultImages[i] retain];
			if (defaultImages[i] == nil) {
				NSLog(@"Failed to load %d.png", i);
				return 0;
			}	
		}
		glGenTextures(6, texture);
		[self initImage];
		photos = [[NSMutableArray alloc] init];
		angle = 0;
		curtImageIndex[0] = 0;
		curtImageIndex[1] = 1;
		movingPhase = movingPhase_Idle;		
		xAxisRotateAngle = 0;
		xAxisRotateDeltaAngle = 0.2;
	}
	return self;
}

- (void)dealloc {
	for (int i = 0; i < 6; i++) {
		[defaultImages[i] release];
	}
	glDeleteTextures(6, texture);
	[photos release];
	[super dealloc];
}

//	平面を構成する頂点
static const GLfloat vertices[][3] = {
	{-1.0, 1.0, 1.0}, 
	{1.0, 1.0, 1.0}, 
	{-1.0, -1.0, 1.0}, 
	{1.0, -1.0, 1.0}
};

//	法線ベクトル
static const GLfloat normals[][3] = {				
	{0.0, 0.0, 1.0},
	{0.0, 0.0, 1.0},
	{0.0, 0.0, 1.0},
	{0.0, 0.0, 1.0}
};

//	平面へのテクスチャのマップ座標
static const GLfloat squareTexCoords[][2] = {
	{0.0, 0.0},
	{1.0, 0.0},
	{0.0, 1.0},
	{1.0, 1.0}
};

//	6面分の回転指定
static const GLfloat rotates[][4] = {
	{0, 0.0, 1.0, 0.0},
	{90, 0.0, 1.0, 0.0},
	{180, 0.0, 1.0, 0.0},
	{-90, 0.0, 1.0, 0.0},
	{90, 1.0, 0.0, 0.0},
	{-90, 1.0, 0.0, 0.0}
};

/*
	平面を描画する。
*/
-(void)drawPlane:(int)part {
	glPushMatrix(); 
	if (part > 0)
		glRotatef(rotates[part][0], rotates[part][1], rotates[part][2], rotates[part][3]); 

	glEnable(GL_TEXTURE_2D);

	glTexCoordPointer(2, GL_FLOAT, 0, squareTexCoords);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);	
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);	//	GL_CLAMP_TO_EDGEだと繰り返さない。
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	if (maxIndex == 0) {
		//	サイコロ描画
		glBindTexture(GL_TEXTURE_2D, texture[part]);
	} else {
		glBindTexture(GL_TEXTURE_2D, texture[(part % 2)]);
	}
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glDisable(GL_TEXTURE_2D);
	glPopMatrix(); 
}

//	サイコロを振る。ただし振って動いている最中なら止める。
-(void)throwDice:(double)inDelta {
	if (movingPhase == movingPhase_Rotate) {
		movingPhase = movingPhase_Idle;
		return;
	}
	movingPhase = movingPhase_Rotate;
}

//	回転
-(void)rotation {
	angle += 10;
}


/*
	描画
*/
-(void)draw {
	//	画面クリア
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClearDepthf(1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);

	glPushMatrix(); 
	if (movingPhase == movingPhase_Idle) {
		//	上下にゆらす
		xAxisRotateAngle += xAxisRotateDeltaAngle;
		if (xAxisRotateAngle > 15) {
			xAxisRotateDeltaAngle = -xAxisRotateDeltaAngle;
			xAxisRotateAngle = 15;
		}
		if (xAxisRotateAngle < -15) {
			xAxisRotateDeltaAngle = -xAxisRotateDeltaAngle;
			xAxisRotateAngle = -15;
		}
		glRotatef(xAxisRotateAngle, 1, 0, 0);
		glRotatef(45, 0, 1, 0);
	} else {
		//	回転
		glRotatef(angle, 0, 1, 0);
	}
	if (maxIndex > 0) {
		//	左側テクスチャのチェック
		int image_index = (int)angle / 90;				//	左側画像番号
		int tx_index = image_index % 2;					//	左側textureのインディックス（image_indexの最大サイズ調整前に計算）
#ifdef CORRECT
		image_index = image_index % maxIndex;			//	ただし最大サイズを超えない。		
		if (curtImageIndex[tx_index] != image_index) {	//	画像が違うなら更新
			[self map:[photos objectAtIndex:image_index] texture:texture[tx_index]];
			curtImageIndex[tx_index] = image_index;
		}
		//	右側テクスチャのチェック
		tx_index = 1 ^ tx_index;						//	右側textureのインディックス
		image_index++;									//	変更する画像番号
		image_index = image_index % maxIndex;			//	ただし最大サイズを超えない。
		if (curtImageIndex[tx_index] != image_index) {	//	画像が違うなら更新
			[self map:[photos objectAtIndex:image_index] texture:texture[tx_index]];
			curtImageIndex[tx_index] = image_index;
		}
#else
		tx_index = 1 ^ tx_index;						//	右側textureのインディックス
		image_index++;									//	右側画像番号
		image_index = image_index % maxIndex;			//	ただし最大サイズを超えない。
		if (curtImageIndex[tx_index] != image_index) {	//	画像が違うなら更新
			[self map:[photos objectAtIndex:image_index] texture:texture[tx_index]];
			curtImageIndex[tx_index] = image_index;
		}
#endif
	}
	if (movingPhase == movingPhase_Rotate) 
		[self rotation];
	#define RGBAVALUE 1.0f
	GLfloat material_amb[4]={RGBAVALUE, RGBAVALUE, RGBAVALUE, 1.0f};	//	素材側環境光特性	RGBA
	GLfloat material_dif[4]={RGBAVALUE, RGBAVALUE, RGBAVALUE, 1.0f};	//	素材側拡散光特性	RGBA
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material_amb);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_dif);
	//	頂点指定
	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, GL_FLOAT, 0, vertices);
    glEnableClientState(GL_NORMAL_ARRAY);
    glNormalPointer(GL_FLOAT, 0, normals);
	//	六面体描画
	for (int i = 0; i < 6; i++) 
		[self drawPlane:i];

	glPopMatrix(); 
}

//	写真設定
-(void)setImages:(UIImage**)images max:(int)max {
	movingPhase = movingPhase_Idle;
	angle = 45;
	curtImageIndex[0] = 0;
	curtImageIndex[1] = 1;
	[photos removeAllObjects];
	[self initImage];
	maxIndex = max;
	for (int i = 0; i < maxIndex; i++) {
		[photos addObject:images[i]];
		if (i < 4) {
			[self map:images[i] texture:texture[i]];
		}
	}
	if (maxIndex == 1) {
		[self map:[photos objectAtIndex:0] texture:texture[1]];
	}
}

@end


